@@ -181,14 +181,10 @@ struct _ViewerCbInfo {
 static cairo_surface_t *
 render_to_surface (ViewerCbInfo *info)
 {
-    RsvgDimensionData dimensions;
     int width, height;
     cairo_matrix_t matrix;
-    GdkPixbuf *output = NULL;
-    guint8 *pixels;
     cairo_surface_t *surface;
     cairo_t *cr;
-    int rowstride, minimum;
 
     width = ceil ((double) info->dimensions.width * info->x_zoom);
     height = ceil ((double) info->dimensions.height * info->y_zoom);
@@ -212,6 +208,12 @@ render_to_surface (ViewerCbInfo *info)
 
     cairo_destroy (cr);
 
+    if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS) {
+        g_printerr ("Error while rendering image: %d\n", cairo_surface_status (surface));
+        cairo_surface_destroy (surface);
+        return NULL;
+    }
+
     return surface;
 }
 
@@ -625,8 +627,6 @@ main (int argc, char **argv)
     int from_stdin = 0;
     ViewerCbInfo info;
 
-    struct RsvgSizeCallbackData size_data;
-
     char **args = NULL;
     gint n_args = 0;
 
@@ -784,11 +784,6 @@ main (int argc, char **argv)
         g_printerr ("Unknown error while rendering image\n");
         goto done;
     }
-    if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS) {
-        g_printerr ("Error while rendering image: %d\n", cairo_surface_status (surface));
-        cairo_surface_destroy (surface);
-        goto done;
-    }
 
     retval = 0;
 
